;; Generated package description from altcaps.el  -*- no-byte-compile: t -*-
(define-package "altcaps" "1.1.0.0.20221128.51256" "Apply alternating letter casing to convey sarcasm or mockery" '((emacs "27.1")) :commit "05b78cc81b8ca92aebb80779c2a863fa5eb18eaf" :authors '(("Protesilaos Stavrou" . "info@protesilaos.com")) :maintainer '("Altcaps Development" . "~protesilaos/altcaps@lists.sr.ht") :url "https://git.sr.ht/~protesilaos/altcaps")
